﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using Bitrix.Components;
using Bitrix.Security;
using System.Collections.Generic;
using Bitrix.Modules;
using Bitrix.UI;
using Bitrix.Sale.SaleCart;
using Bitrix.DataLayer;
using Bitrix.DataTypes;
using System.Web;
using Bitrix.Services.Text;
using Bitrix.Sale;
using Bitrix.Services;

namespace Bitrix.Sale.Components
{
	public enum SaleOrderCancelComponentError
	{
		None = 0,
		Unauthorized,
		OrderNotFound,
		OrderIsAlreadyCancelled
	}

	public partial class SaleOrderCancelComponent : BXComponent
	{
		SaleOrderCancelComponentError error = SaleOrderCancelComponentError.None;

		public SaleOrderCancelComponentError Error
		{
			get { return error; }
		}

		public string GetErrorMessage()
		{
			if (error == SaleOrderCancelComponentError.None)
				return string.Empty;

			return GetMessage("Error." + error.ToString());
		}

		int curUserId = -1;

		//Id текущего пользователя, у анонимов - 0
		public int CurrentUserId
		{
			get
			{
				return curUserId != -1 ? curUserId : (curUserId =
					(BXPrincipal.Current != null && BXPrincipal.Current.Identity.IsAuthenticated ? ((BXIdentity)BXPrincipal.Current.Identity).Id : 0));
			}
		}

		BXSaleOrder order;

		public int OrderId
		{
			get { return Parameters.GetInt("OrderId", 0); }
			set { Parameters["OrderId"] = value.ToString(); }
		}

		public string OrderDetailUrlTemplate
		{
			get { return Parameters.GetString("OrderDetailUrlTemplate", "~/orders.aspx?order=#OrderId#"); }
			set { Parameters["OrderDetailUrlTemplate"] = value; }
		}

		public string OrderListUrlTemplate
		{
			get { return Parameters.GetString("OrderListUrlTemplate", "~/orderlist.aspx"); }
			set { Parameters["OrderListUrlTemplate"] = value; }
		}

		protected void Page_Load(object sender, EventArgs e)
		{
			if (CurrentUserId == 0)
				error = SaleOrderCancelComponentError.Unauthorized;

			if (OrderId == 0)
				error = SaleOrderCancelComponentError.OrderNotFound;

			var order = BXSaleOrder.GetById(OrderId);
			if (order == null)
				error = SaleOrderCancelComponentError.OrderNotFound;

			if (order!= null && order.Cancelled)
				error = SaleOrderCancelComponentError.OrderIsAlreadyCancelled;

			if (Parameters.Get<bool>("SetPageTitle", true) && OrderId > 0)
			{
				((BXPublicPage)Page).BXTitle = String.Format(GetMessage("Title.CancelOrder"), OrderId);
			}

			var replace = new BXParamsBag<object>();
			replace.Add("UserId", CurrentUserId);
			orderListUrl = ResolveTemplateUrl(OrderListUrlTemplate, replace);
			replace.Add("OrderId", OrderId);
			orderDetailUrl = ResolveTemplateUrl(OrderDetailUrlTemplate, replace);
			
			IncludeComponentTemplate();
		}

		public override bool ProcessCommand(string commandName, BXParamsBag<object> commandParameters, List<string> commandErrors)
		{
			if (commandName.Equals("CancelOrder", StringComparison.OrdinalIgnoreCase))
			{

				BXSaleOrder order = null;
				if (OrderId == 0 || (order = BXSaleOrder.GetById(OrderId)) == null)
				{
					commandErrors.Add(GetMessage("Error.OrderNotFound"));
					return false;
				}
				BXSaleOrderCancelReason reason = BXSaleOrderCancelReason.DecisionChanged;
				string strReason;
				bool reasonIsSet = false;

				if( commandParameters.TryGetValue<string>("reason", out strReason))
				{
					if (Enum.IsDefined(typeof(BXSaleOrderCancelReason), strReason))
					{
						reason = (BXSaleOrderCancelReason)Enum.Parse(typeof(BXSaleOrderCancelReason), strReason);
						reasonIsSet = true;
					}
				}

				if (!reasonIsSet)
				{
					commandErrors.Add(GetMessage("Error.WrongCancelReason"));
					return false;
				}

			var products = BXSaleProduct.GetList(new BXFilter(new BXFilterItem(BXSaleProduct.Fields.OrderId, BXSqlFilterOperators.Equal, order.Id)),null);
				if(products.Count > 0 )
				{
					var errors = new List<string>();
					try
					{
						using(var helper = new BXSqlTransactionHelper(null))
						{
							BXSaleCartItemManager.UpdateBalance(
							products.ConvertAll<BXSaleBalanceInfo>(
											x => new BXSaleBalanceInfo()
											{
												ItemId = x.CartElementId,
												Quantity = x.Quantity
											}).ToArray(),
										products[0].CartResolverId, helper, errors);
							helper.Commit();
						}
					}
					catch (Exception e)
					{
						BXLogService.LogAll(new BXLogMessage(GetMessage("Error.UpdateBalance"), e.Message, BXLogMessageType.Error));
					}
				}
				order.ReasonCancelled = reason;
				order.Cancelled = true;
				order.Save();
				using(var helper = new BXSqlTransactionHelper(null))
				{
					BXSaleDiscountCoupon.ActivateCouponsForOrder(helper, order.Id);
					helper.Commit();
				}
			}
			return true;
		}

		protected override void PreLoadComponentDefinition()
		{
			base.Title = GetMessage("Component.Title");
			base.Description = GetMessage("Component.Description");
			base.Icon = "images/sale_order_list.gif";

			Group = new BXComponentGroup("Store", GetMessage("Group"), 100, BXComponentGroup.Store);

			ParamsDefinition.Add(
				"OrderId",
				new BXParamText(
					GetMessageRaw("Param.OrderId"),
					"",
					BXCategory.Main
					)
				);

			ParamsDefinition.Add(
				"SetPageTitle",
				new BXParamYesNo(
					GetMessageRaw("Param.SetPageTitle"),
					true,
					BXCategory.Main
					)
				);

			ParamsDefinition.Add(
				"OrderListUrlTemplate",
				new BXParamText(
					GetMessageRaw("Param.OrderListUrlTemplate"),
					"~/orderlist.aspx",
					BXCategory.UrlSettings
					)
				);

			ParamsDefinition.Add(
				"OrderDetailUrlTemplate",
				new BXParamText(
					GetMessageRaw("Param.OrderDetailUrlTemplate"),
					"~/orderdetail.aspx?orderId=#OrderId#",
					BXCategory.UrlSettings
					)
				);
		}

		IDictionary<string, string> cancelReasons;
		public IDictionary<string, string> CancelReasons
		{
			get
			{
				if (cancelReasons != null)
					return cancelReasons;
				cancelReasons = new Dictionary<string, string>();
				foreach (BXSaleOrderCancelReason val in Enum.GetValues(typeof(BXSaleOrderCancelReason)))
				{
					cancelReasons[val.ToString()] = BXSaleOrderCancelReasonLocalization.GetLocalizedName(val);
				}
				//cancelReasons.Sort(delegate(string s1, string s2) { return s1.CompareTo(s2); });
				return cancelReasons;
			}
		}

		string orderDetailUrl;
		public string OrderDetailUrl
		{
			get
			{
				return orderDetailUrl ?? String.Empty;
			}
		}

		string orderListUrl;
		public string OrderListUrl
		{
			get
			{
				return orderListUrl ?? String.Empty;
			}
		}
	}
}

	