﻿using System;
using System.Data;
using System.Configuration;
using System.Collections;
using Bitrix.Components;
using Bitrix.Security;
using System.Collections.Generic;
using Bitrix.Modules;
using Bitrix.UI;
using Bitrix.Sale.SaleCart;
using Bitrix.DataLayer;
using Bitrix.DataTypes;
using System.Web;
using Bitrix.Services.Text;
using Bitrix.Sale;
using Bitrix.Currency;
using Bitrix.Components.Editor;

namespace Bitrix.Sale.Components
{
	public enum SaleOrderDetailComponentError
	{
		None=0,
		Unauthorized,
		OrderNotFound
	}

	public partial class SaleOrderDetailComponent : BXComponent
	{
		int curUserId = -1;

		SaleOrderDetailComponentError error = SaleOrderDetailComponentError.None;

		public SaleOrderDetailComponentError Error
		{
			get { return error; }
			set { error = value; }
		}

		//Id текущего пользователя, у анонимов - 0
		public int CurrentUserId
		{
			get
			{
				return curUserId != -1 ? curUserId : (curUserId =
					(BXPrincipal.Current != null && BXPrincipal.Current.Identity.IsAuthenticated ? ((BXIdentity)BXPrincipal.Current.Identity).Id : 0));
			}
		}

		BXUser user;

		public BXUser User
		{
			get
			{
				if ( CurrentUserId == 0 )
					return null;
				return user ?? (user = BXUser.GetById(CurrentUserId));  
			}
		}

		public int OrderId
		{
			get { return Parameters.GetInt("OrderId", 0); }
			set { Parameters["OrderId"] = value.ToString(); }
		}

		BXSaleOrder order;

		public BXSaleOrder Order
		{
			get { return order; }
		}

		List<BXSaleProduct> products;

		public IList<BXSaleProduct> Products
		{
			get { return products ?? (products = new List<BXSaleProduct>()); }
		}

		public string GetErrorMessage()
		{
			return GetMessage("Error." + error.ToString());
		}

		public string OrderStatusLocalizedName
		{
			get { 
				string name;
				if( Order.Status == null || String.IsNullOrEmpty((name = Order.Status.GetLocalizedName(Bitrix.Services.BXLoc.CurrentLocale))))
					return GetMessage("StatusNameNotFound");

				return name;
			}
		}

		public string DeliveryServiceName
		{
			get
			{
				string name;
				if (BXSaleDeliveryManager.TryGetVariantDescription(order.DeliveryVariant, out name))
					return name;
				return GetMessage("Label.DeliveryServiceNotSet");
			}
		}

		public string PaymentSystemName
		{
			get { return Order.PaymentSystem != null ? Order.PaymentSystem.Name : string.Empty; }
		}

		public string FormattedOrderSum
		{
			get { return Currency.MoneyToString(Order.GetConvertedSum(Currency)); }
		}

		public string FormattedOrderDeliveryCost
		{
			get { return Currency.MoneyToString(Order.Currency.ConvertMoney(Order.DeliveryCost, Currency.Id)); }
		}

		bool TryLoadOrder()
		{
			var orders = BXSaleOrder.GetList(new BXFilter(
				new BXFilterItem(BXSaleOrder.Fields.Id, BXSqlFilterOperators.Equal, OrderId),
				new BXFilterItem(BXSaleOrder.Fields.UserId, BXSqlFilterOperators.Equal, CurrentUserId)
				), 
                null,
                new BXSelectAdd(BXSaleOrder.Fields.PaymentSystem),
                new BXQueryParams(new BXPagingOptions(0, 1)));

			if (orders.Count == 0)
			{
				Error = SaleOrderDetailComponentError.OrderNotFound;
				return false;
			}

			order = orders[0];
			return true;
		}

		protected void Page_Load(object sender, EventArgs e)
		{
			if (CurrentUserId == 0)
			{
				Error = SaleOrderDetailComponentError.Unauthorized;
				IncludeComponentTemplate();
				return;
			}

			if (OrderId == 0)
			{
				Error = SaleOrderDetailComponentError.OrderNotFound;
				IncludeComponentTemplate();
				return;
			}

			if (Parameters.Get<bool>("SetPageTitle",true))
			{
				((BXPublicPage)Page).BXTitle = String.Format(GetMessage("Title.MyOrder"), OrderId);
			}

			if (!TryLoadOrder())
			{
				IncludeComponentTemplate();
				return;
			}
			
			products = BXSaleProduct.GetList(
				new BXFilter(new BXFilterItem(BXSaleProduct.Fields.OrderId, BXSqlFilterOperators.Equal, OrderId)),
				new BXOrderBy(new BXOrderByPair(BXSaleProduct.Fields.Id, BXOrderByDirection.Asc)));

			var replace = new BXParamsBag<object>();
			replace.Add("UserId", CurrentUserId);

			orderListUrl = ResolveTemplateUrl(OrderListUrlTemplate, replace); 

			IncludeComponentTemplate();
		}

		public BXCurrency Currency
		{
			get { return BXCurrency.NationalCurrency; }
		}

		public decimal GetTotalWeight()
		{
			if (products == null || products.Count == 0)
				return 0m;

			decimal r = 0m;
			for (int i = 0; i < products.Count; i++)
				r += products[i].Weight;

			return r;
		}

		public string OrderListUrlTemplate
		{
			get { return Parameters.GetString("OrderListUrlTemplate", "~/orders.aspx"); }
			set { Parameters["OrderListUrlTemplate"] = value; }
		}

		string orderListUrl;

		public string OrderListUrl
		{
			get { return orderListUrl; }
		}

		public List<string> DisplayCustomPropsCodes
		{
			get
			{
				if (Order != null)
					return Parameters.GetListString("DisplayCustomProps_" + Order.CustomerType.Code.ToString());
				else return new List<string>();
			}
		}

		public List<int> DisplayCustomProps
		{
			get {
				if (Order != null)
					return Parameters.GetListInt("DisplayCustomProps_" + Order.CustomerType.Code.ToString());
				else return new List<int>();
			}
		}

		protected override void PreLoadComponentDefinition()
		{
			base.Title = GetMessage("Component.Title");
			base.Description = GetMessage("Component.Description");
			base.Icon = "images/sale_order_detail.gif";
			base.Group = new BXComponentGroup("Store", GetMessage("Group"), 100, BXComponentGroup.Store);

			ParamsDefinition.Add(
				"OrderId",
				new BXParamText(
					GetMessageRaw("Param.OrderId"),
					"<%$ Request:orderid %>",
					BXCategory.Main
					)
				);

			ParamsDefinition.Add(
				"SetPageTitle",
				new BXParamYesNo(
					GetMessageRaw("Param.SetPageTitle"),
					true,
					BXCategory.Main
					)
				);
			foreach (var customerType in CustomerTypes)
			{
				ParamsDefinition.Add(
					"DisplayCustomProps_"+customerType.Code.ToString(),
					new BXParamDoubleList(
						String.Format(GetMessage("Param.DisplayProps"), customerType.TextEncoder.Decode(customerType.Name)),
						"",
						BXCategory.Main
						)
					);
			}

			ParamsDefinition.Add(
				"OrderListUrlTemplate",
				new BXParamText(
					GetMessageRaw("Param.OrderListUrlTemplate"),
					"~/orders.aspx",
					BXCategory.UrlSettings
					)
				);
		}

		BXSaleCustomerTypeCollection cTypes;

		protected BXSaleCustomerTypeCollection CustomerTypes
		{
			get { return cTypes ?? (cTypes = BXSaleCustomerType.GetAllForSite(DesignerSite)); }
		}

		protected override void LoadComponentDefinition()
		{
			foreach (var customerType in CustomerTypes)
			{
				ParamsDefinition["DisplayCustomProps_" + customerType.Code.ToString()]
					.Values.AddRange( 
					BXSaleOrderField.GetCustomerFields(customerType.Id, null).ConvertAll<BXParamValue>(
						x => new BXParamValue(x.Name, x.Code.ToString())
						));
			}
		}

		public Dictionary<int, CustomProperty> CustomProperties
		{
			get
			{
				var result = new Dictionary<int, CustomProperty>();
				if (order == null)
					return result;
				var fields = BXSaleOrderField.GetCustomerFields(order.CustomerTypeId, null);
				var values = BXSaleOrderValueManager.GetValues(order.Id, fields, null);

				foreach (var v in values.Values)
				{
					var f = fields[v.Name];
					if (f != null)
					{
						var view = f.CustomType.CreatePublicView();
						view.Init(v, f.GetCustomFieldAdapter());
						result.Add(f.Id, new CustomProperty
						{
							Title = f.Name,
							Html = view.GetHtml("<br/>"),
							Sort = f.Sort
						});
					}
				}
				//result.Sort((x, y) => x.Sort - y.Sort);
				return result;
			}
		}
		public Dictionary<string, CustomProperty> StrCustomProperties
		{
			get
			{
				var result = new Dictionary<string, CustomProperty>();
				if (order == null)
					return result;
				var fields = BXSaleOrderField.GetCustomerFields(order.CustomerTypeId, null);
				var values = BXSaleOrderValueManager.GetValues(order.Id, fields, null);

				foreach (var v in values.Values)
				{
					var f = fields[v.Name];
					if (f != null)
					{
						var view = f.CustomType.CreatePublicView();
						view.Init(v, f.GetCustomFieldAdapter());
						result.Add(f.Code, new CustomProperty
						{
							Title = f.Name,
							Html = view.GetHtml("<br/>"),
							Sort = f.Sort
						});
					}
				}
				//result.Sort((x, y) => x.Sort - y.Sort);
				return result;
			}
		}
		public class CustomProperty
		{
			public string Title { get; set; }
			public string Html { get; set; }
			public int Sort { get; set; }
		}
	}


}
