﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Web.UI;
using System.Web.UI.WebControls;
using Bitrix.Sale;
using Bitrix.DataLayer;
using Bitrix.Currency;
using System.Globalization;

/// <summary>
/// Обработчик оплаты заказа "Счёт"
/// </summary>
public partial class BXBillPaymentController : UserControl, IBXSalePaymentController
{
    #region IBXSalePaymentController Members
    public string DisplayName
    {
        get { return BXSaleModule.GetMessage("BillPaymentController.DisplayName", false); }
    }

    private BXSaleOrder order = null;
    private BXSalePaymentSystemConfig conf = null;
    private BXSalePaymentControllerSettings settings = null;
    public void InitializeData(BXSaleOrder order, BXSalePaymentSystemConfig conf)
    {
        if(order == null)
            throw new ArgumentNullException("order");

        this.order = order;
        this.products = BXSaleProduct.GetList(
            new BXFilter(
                new BXFilterItem(BXSaleProduct.Fields.OrderId, BXSqlFilterOperators.Equal, order.Id)
                ),
            new BXOrderBy(new BXOrderByPair(BXSaleProduct.Fields.Id, BXOrderByDirection.Asc))
            );

        this.currency = order.Currency;
        this.conf = conf;
        if (conf != null)
        {
            this.settings = conf.ControllerSettings;
            this.settings.Order = order;
        }
    }
    #endregion

    public int OrderId
    {
        get { return this.order.Id; }
    }

    private readonly static string subsForEmpty = "-";

    private string GetHtmlEncodeStringParam(string name, string defaultValue)
    {
        if (this.settings == null)
            return defaultValue;

        return HttpUtility.HtmlEncode(this.settings.GetString(name, defaultValue));
    }

    private string GetStringParam(string name, string defaultValue)
    {
        if (this.settings == null)
            return defaultValue;

        return this.settings.GetString(name, defaultValue);
    }

    /// <summary>
    /// Дата создания заказа
    /// </summary>
    public string OrderCreationDate
    {
        get
        {
            if (this.settings == null)
                return subsForEmpty;

            DateTime d = this.settings.GetDateTime("orderCreationDate", DateTime.MinValue);
            return d != DateTime.MinValue ? HttpUtility.HtmlEncode(d.ToString("d.mm.yyyy")) : subsForEmpty;
        }
    }

    /// <summary>
    /// Поставщик
    /// </summary>
    public string SellerName
    {
        get { return GetHtmlEncodeStringParam("sellerName", string.Empty); }
    }

    /// <summary>
    /// Адрес поставщика
    /// </summary>
    public string SellerAddress
    {
        get { return GetHtmlEncodeStringParam("sellerAddress", string.Empty); }
    }

    /// <summary>
    /// Телефон поставщика
    /// </summary>
    public string SellerPhone
    {
        get { return GetHtmlEncodeStringParam("sellerPhone", string.Empty); }
    }

    /// <summary>
    /// ИНН поставщика
    /// </summary>
    public string SellerINN
    {
        get { return GetHtmlEncodeStringParam("sellerInn", string.Empty); }
    }

    /// <summary>
    /// KPP поставщика
    /// </summary>
    public string SellerKPP
    {
        get { return GetHtmlEncodeStringParam("sellerKpp", string.Empty); }
    }

    /// <summary>
    /// Расчетный счёт поставщика
    /// </summary>
    public string SellerSettlementAccount
    {
        get { return GetHtmlEncodeStringParam("sellerSettlementAccount", string.Empty); }
    }

    /// <summary>
    /// Корсчёт банка поставщика
    /// </summary>
    public string SellerCorrespondentAccount
    {
        get { return GetHtmlEncodeStringParam("sellerCorrespondentAccount", string.Empty); }
    }

    /// <summary>
    /// БИК банка поставщика
    /// </summary>
    public string SellerBIC
    {
        get { return GetHtmlEncodeStringParam("sellerBic", string.Empty); }
    }

    /// <summary>
    /// Наименование покупателя
    /// </summary>
    public string BuyerName
    {
        get { return GetHtmlEncodeStringParam("buyerName", string.Empty); }
    }

    /// <summary>
    /// ИНН покупателя
    /// </summary>
    public string BuyerINN
    {
        get { return GetHtmlEncodeStringParam("buyerInn", string.Empty); }
    }

    /// <summary>
    /// Адрес покупателя
    /// </summary>
    public string BuyerAddress
    {
        get { return GetHtmlEncodeStringParam("buyerAddress", string.Empty); }
    }

    /// <summary>
    /// Телефон поставщика
    /// </summary>
    public string BuyerPhone
    {
        get { return GetHtmlEncodeStringParam("buyerPhone", string.Empty); }
    }

    /// <summary>
    /// Факс поставщика
    /// </summary>
    public string BuyerFax
    {
        get { return GetHtmlEncodeStringParam("buyerFax", string.Empty); }
    }

    /// <summary>
    /// Факс поставщика
    /// </summary>
    public string BuyerContactPerson
    {
        get { return GetHtmlEncodeStringParam("buyerContactPerson", string.Empty); }
    }

    /// <summary>
    /// Имя системы оплаты
    /// </summary>
    public string PaymentSystemName
    {
        get { return this.conf != null ? this.conf.Name : subsForEmpty; }
    }

    public decimal OrderTotal
    {
        get { return this.order != null ? this.order.Sum : 0m; }
    }

    private decimal? vatSum = null;
    public decimal VATSum
    {
        get
        {
            if (this.vatSum.HasValue)
                return this.vatSum.Value;

            decimal m = 0m;
            for (int i = 0; i < Products.Count; i++)
                m += Products[i].GetVatSum();

            return (this.vatSum = m).Value;
        }
    }

    public string CurrencySymbol
    {
        get { return this.currency != null ? this.currency.GetSymbol() : string.Empty; }
    }

    private BXSaleProductCollection products = null;
    public IList<BXSaleProduct> Products
    {
        get { return products; }
    }

    public decimal DeliveryCost
    {
        get { return this.order != null ? this.order.DeliveryCost : 0m; }
    }

    public string DeliveryVariantDescription
    {
        get
        {
            if (this.order == null)
                return string.Empty;

            string s;
            BXSaleDeliveryManager.TryGetVariantDescription(this.order.DeliveryVariant, out s);
            return HttpUtility.HtmlEncode(s);
        }
    }

    private BXCurrency currency = null;
    public BXCurrency Currency
    {
        get { return this.currency; }
    }

    public string FormatMoney(decimal value)
    {
        return this.currency != null ? this.currency.MoneyToString(value, CultureInfo.CurrentCulture, false) : value.ToString("F2");
    }

    private string pathToStamp = null;
    public string PathToStamp
    {
        get
        {
            if (this.pathToStamp != null)
                return this.pathToStamp;

            this.pathToStamp = GetStringParam("pathToStamp", string.Empty);
            if (this.pathToStamp.Length > 0)
                this.pathToStamp = VirtualPathUtility.ToAbsolute(this.pathToStamp);

            return this.pathToStamp;
        }
    }
}
